==> opik-0.1.1.tar.gz: opik-0.1.1/src/opik/evaluation/metrics/llm_judges/hallucination/template.py <==
from typing import List, TypedDict

HALLUCINATION_VERDICT = "hallucinated"
FACTUAL_VERDICT = "factual"

VERDICT_KEY = "verdict"
REASON_KEY = "reason"


class FewShotExampleHallucination(TypedDict):
    title: str
    input: str
    context: List[str]
    output: str
    verdict: str
    reason: str


def generate_query(
    input: str,
    output: str,
    context: List[str],
    few_shot_examples: List[FewShotExampleHallucination],
) -> str:
    if len(few_shot_examples) == 0:
        examples_str = ""
    else:
        examples_str = "\n\nEXAMPLES:\n\n".join(
            [
                f"<example>\n"
                f"Input: {example['input']}\n"
                f"Context: {example['context']}\n"
                f"Output: {example['output']}\n\n"
                f"{{\"{VERDICT_KEY}\": \"{example['verdict']}\", \"{REASON_KEY}\": \"{example['reason']}\"}}\n"
                f"</example>"
                for i, example in enumerate(few_shot_examples)
            ]
        )

    return f"""You are an expert judge tasked with evaluating the faithfulness of an AI-generated answer to the given context. Analyze the provided INPUT, CONTEXT, and OUTPUT to determine if the OUTPUT contains any hallucinations or unfaithful information.

Guidelines:
1. The OUTPUT must not introduce new information beyond what's provided in the CONTEXT.
2. The OUTPUT must not contradict any information given in the CONTEXT.
3. Ignore the INPUT when evaluating faithfulness; it's provided for context only.
4. Consider partial hallucinations where some information is correct but other parts are not.
5. Pay close attention to the subject of statements. Ensure that attributes, actions, or dates are correctly associated with the right entities (e.g., a person vs. a TV show they star in).
6. Be vigilant for subtle misattributions or conflations of information, even if the date or other details are correct.
7. Check that the OUTPUT doesn't oversimplify or generalize information in a way that changes its meaning or accuracy.

Verdict options:
- "{FACTUAL_VERDICT}": The OUTPUT is entirely faithful to the CONTEXT.
- "{HALLUCINATION_VERDICT}": The OUTPUT contains hallucinations or unfaithful information.

{examples_str}

INPUT (for context only, not to be used for faithfulness evaluation):
{input}

CONTEXT:
{context}

OUTPUT:
{output}

Provide your verdict in JSON format:
{{
    "{VERDICT_KEY}": <your verdict>,
    "{REASON_KEY}": [
        <list your reasoning as bullet points>
    ]
}}
    """

==> opik-1.0.1.tar.gz: opik-1.0.1/src/opik/evaluation/metrics/llm_judges/hallucination/template.py <==
from typing import List, TypedDict, Optional

HALLUCINATION_VERDICT = "hallucinated"
FACTUAL_VERDICT = "factual"

VERDICT_KEY = "verdict"
REASON_KEY = "reason"


class FewShotExampleHallucination(TypedDict):
    title: str
    input: str
    context: List[str]
    output: str
    verdict: str
    reason: str


context_hallucination_template = """You are an expert judge tasked with evaluating the faithfulness of an AI-generated answer to the given context. Analyze the provided INPUT, CONTEXT, and OUTPUT to determine if the OUTPUT contains any hallucinations or unfaithful information.

Guidelines:
1. The OUTPUT must not introduce new information beyond what's provided in the CONTEXT.
2. The OUTPUT must not contradict any information given in the CONTEXT.
2. The OUTPUT should not contradict well-established facts or general knowledge.
3. Ignore the INPUT when evaluating faithfulness; it's provided for context only.
4. Consider partial hallucinations where some information is correct but other parts are not.
5. Pay close attention to the subject of statements. Ensure that attributes, actions, or dates are correctly associated with the right entities (e.g., a person vs. a TV show they star in).
6. Be vigilant for subtle misattributions or conflations of information, even if the date or other details are correct.
7. Check that the OUTPUT doesn't oversimplify or generalize information in a way that changes its meaning or accuracy.

Verdict options:
- "{FACTUAL_VERDICT}": The OUTPUT is entirely faithful to the CONTEXT.
- "{HALLUCINATION_VERDICT}": The OUTPUT contains hallucinations or unfaithful information.

{examples_str}

INPUT (for context only, not to be used for faithfulness evaluation):
{input}

CONTEXT:
{context}

OUTPUT:
{output}

Provide your verdict in JSON format:
{{
    "{VERDICT_KEY}": <your verdict>,
    "{REASON_KEY}": [
        <list your reasoning as bullet points>
    ]
}}"""

output_hallucination_template = """You are an expert judge tasked with evaluating the factual accuracy and reliability of an AI-generated answer. Analyze the provided INPUT, and OUTPUT to determine if the OUTPUT contains any hallucinations or unfaithful information.

Guidelines:
1. Evaluate the OUTPUT based on generally accepted facts and reliable information.
2. The OUTPUT should not contradict well-established facts or general knowledge.
3. Ignore the INPUT when evaluating faithfulness; it's provided for context only.
4. Consider partial hallucinations where some information is correct but other parts are not.
5. Pay close attention to the subject of statements. Ensure that attributes, actions, or dates are correctly associated with the right entities (e.g., a person vs. a TV show they star in).
6. Be vigilant for subtle misattributions or conflations of information, even if the date or other details are correct.
7. Check that the OUTPUT doesn't oversimplify or generalize information in a way that changes its meaning or accuracy.

Verdict options:
- "{FACTUAL_VERDICT}": The OUTPUT does not contain any hallucinations or unfaithful information.
- "{HALLUCINATION_VERDICT}": The OUTPUT contains hallucinations or unfaithful information.

{examples_str}

INPUT (for context only, not to be used for faithfulness evaluation):
{input}

OUTPUT:
{output}

Provide your verdict in JSON format:
{{
    "{VERDICT_KEY}": <your verdict>,
    "{REASON_KEY}": [
        <list your reasoning as bullet points>
    ]
}}"""


def generate_query(
    input: str,
    output: str,
    context: Optional[List[str]] = None,
    few_shot_examples: Optional[List[FewShotExampleHallucination]] = None,
) -> str:
    if few_shot_examples is None:
        examples_str = ""
    else:
        examples_str = "\n\nEXAMPLES:\n\n".join(
            [
                f"<example>\n"
                f"Input: {example['input']}\n"
                f"Context: {example['context']}\n"
                if context is not None
                else ""
                f"Output: {example['output']}\n\n"
                f"{{\"{VERDICT_KEY}\": \"{example['verdict']}\", \"{REASON_KEY}\": \"{example['reason']}\"}}\n"
                f"</example>"
                for i, example in enumerate(few_shot_examples)
            ]
        )

    if context is not None:
        return context_hallucination_template.format(
            FACTUAL_VERDICT=FACTUAL_VERDICT,
            HALLUCINATION_VERDICT=HALLUCINATION_VERDICT,
            VERDICT_KEY=VERDICT_KEY,
            REASON_KEY=REASON_KEY,
            examples_str=examples_str,
            input=input,
            context=context,
            output=output,
        )
    return output_hallucination_template.format(
        FACTUAL_VERDICT=FACTUAL_VERDICT,
        HALLUCINATION_VERDICT=HALLUCINATION_VERDICT,
        VERDICT_KEY=VERDICT_KEY,
        REASON_KEY=REASON_KEY,
        examples_str=examples_str,
        input=input,
        output=output,
    )
